@@ -1,9 +1,10 @@
-from factory import Factory
+from factory import Factory, Trait
 from contextlib import contextmanager
 from threading import local
 
 # Registry for all factories
 factories = {}
+traits = {}
 
 local = local()
 
@@ -11,7 +12,7 @@ local = local()
 @contextmanager
 def factory(name, collection):
     """Declares a new named factory with the given attributes."""
-    factory = Factory(collection)
+    factory = Factory(collection, global_traits=traits)
     factories[name] = factory
 
     # Set the context for other methods
@@ -35,7 +36,10 @@ def document(name, attrs, parent=None, traits=[]):
 
 
 def trait(name, attrs, parent=None):
-    _get_factory().trait(name, attrs, parent)
+    if hasattr(local, 'working_factory'):
+        _get_factory().trait(name, attrs, parent)
+    else:
+        traits[name] = Trait(attrs, parent)
 
 
 def fragment(name, attrs, parent=None, traits=[]):
@@ -61,16 +65,16 @@ def build(factory, document=None, **overrides):
     return factories[factory].build(document, **overrides)
 
 
-def build_list(number, factory, document=None):
+def build_list(number, factory, document=None, **overrides):
     """Builds a list of `count` instances of the named document using the 
     associated factory."""
-    return [build(factory, document) for x in range(number)]
+    return [build(factory, document, **overrides) for x in range(number)]
 
 
-def create_list(count, factory, document=None):
+def create_list(count, factory, document=None, **overrides):
     """Creates a list of `count` instances of the named document using the 
     associated factory."""
-    return [create(factory, document) for x in range(count)]
+    return [create(factory, document, **overrides) for x in range(count)]
 
 
 # Cleanup methods
@@ -83,7 +87,9 @@ def cleanup():
 def reset():
     """Resets Monufacturer, removing all registered factories. Only really 
     here for testing purposes."""
+    cleanup()
     factories.clear()
+    traits.clear()
 
 
 class FactoryContextException(Exception):
